<!doctype html>
<html>
<head>
    <meta charset="utf-8">
    <script>
        new window.parent.CMS.Sideframe().close();
        window.top.location = '{{ url }}';
    </script>
</head>
<body></body>
</html>
