{% extends "admin/base_site.html" %}
{% load i18n %}

{% load cms_admin cms_static %}

{% block extrahead %}
    {# in case plugins require widgets, they need to have bundle here #}
    <script src="{% static_with_version "cms/js/dist/bundle.admin.base.min.js" %}" type="text/javascript"></script>
    <script type="text/javascript">
        if (!window.jQuery) {
            window.jQuery = CMS.$;
            window.$ = window.jQuery;
        }
    </script>
    {{ block.super }}
    <style>
        .actions-diff-view {
            margin: 0 auto;
            width: 100%;
            border-radius: 4px;
            padding-bottom: 16px;
        }

        ins {
            background-color: #ffffff;
            border: 2px solid #97f295;
            border-radius: 5px;
            text-decoration: none;
            display: inline-block;
            padding: 2px;
        }

        del {
            background-color: #ffffff;
            border: 2px solid #ffb6ba;
            border-radius: 5px;
            text-decoration: line-through;
            display: inline-block;
            padding: 2px;
        }

    </style>
{% endblock %}

{% block content %}
    {% for diff in diffs %}
        <div class="actions-diff-view">
            {% autoescape off %}
                {{ diff }}
            {% endautoescape %}
        </div>
    {% endfor %}
{% endblock content %}
