{% extends 'admin/change_form.html' %}
{% load i18n %}

{% block object-tools %}{% endblock %}

{#    no buttons#}
{% block submit_buttons_bottom %}{% endblock %}

{% block after_field_sets %}
    <table class="workflow-action-table">
        <thead>
        <tr>
            <th>{% trans 'Step' %}</th>
            <th>{% trans 'Stage' %}</th>
            <th>{% trans 'Action' %}</th>
            <th>{% trans 'User' %}</th>
            <th>{% trans 'Message' %}</th>
            <th>{% trans 'Date' %}</th>
        </tr>
        </thead>
        <tbody>
            {% for action in actions %}
                <tr>
                    <td>{{ forloop.counter }}</td>
                    <td>{{ action.stage|default:'&mdash;' }}</td>
                    <td>{{ action.get_action_type_display }}</td>
                    <td>{{ action.user.get_full_name|default:action.user.get_username }}</td>
                    <td>{{ action.message|default:'&mdash;'}}</td>
                    <td>{{ action.created }}</td>
                </tr>
            {% endfor %}
        </tbody>
    </table>
    {% include 'workflows/admin/close_sideframe_link.html' %}
{% endblock %}
